#version 330 core

// Interpolated values from the vertex shaders
in vec2 varyingUV;
in vec3 Normal_worldspace;
in vec3 EyeDirection_cameraspace;
in vec4 ShadowCoord;
in vec3 tangentout;

// Ouput data
layout(location = 0) out vec3 color;

// Values that stay constant for the whole mesh.
//uniform vec3 LightPosition_worldspace;
uniform mat4 MV;
uniform sampler2D texture0;
uniform sampler2D shadowMap;
uniform sampler2D normalMap;
uniform vec3 LightInvDirection;

const float shininess = 20;

vec3 CalcBumpedNormal()                                                                     
{                                                                                           
    vec3 Normal = normalize(Normal_worldspace);                                                       
    vec3 Tangent = normalize(tangentout);                                                     
    Tangent = normalize(Tangent - dot(Tangent, Normal) * Normal);                           
    vec3 Bitangent = cross(Tangent, Normal);                                                
    vec3 BumpMapNormal = texture(normalMap, varyingUV).xyz;                                
    BumpMapNormal = 2.0 * BumpMapNormal - vec3(1.0, 1.0, 1.0);                              
    vec3 NewNormal;                                                                         
    mat3 TBN = mat3(Tangent, Bitangent, Normal);                                            
    NewNormal = TBN * BumpMapNormal;                                                        
    NewNormal = normalize(NewNormal);                                                       
    return NewNormal;                                                                       
}

float ChebychevInequality (vec2 moments, float t) {
	// No shadow if depth of fragment is in front
	if ( t <= moments.x )
		return 1.0;

	// Calculate variance, which is actually the amount of
	// error due to precision loss from fp32 to RG/BA
	// (moment1 / moment2)
	float variance = moments.y - (moments.x * moments.x);
	variance = max(variance, 0.002);
	
	// Calculate the upper bound
	float d = t - moments.x;
	
	return (variance / (variance + d * d));
}

float VsmFixLightBleed (float pMax, float amount) {
	return clamp((pMax - amount) / (1.0 - amount), 0.0, 1.0);
}

void main() {

	
	// Material properties
	vec3 MaterialDiffuseColor = texture( texture0, varyingUV ).rgb;
	vec3 MaterialAmbientColor = 0.15 * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3(0.1,0.1,0.1);

	
	vec3 tmpColor = MaterialAmbientColor;
	// Calculate diffuse term
	vec3 normal = CalcBumpedNormal();//normalize(Normal_worldspace);
	vec3 lightVec = normalize(LightInvDirection);
	float l = dot(normal, lightVec);
	if ( l > 0.0 ) {
			
		// Calculate specular term
		vec3 r = -normalize(reflect(lightVec, normal));
		float s = pow(max(dot(r, EyeDirection_cameraspace), 0.0), shininess);
			
		// Add to color
		tmpColor += ((MaterialDiffuseColor.xyz * l) + (MaterialSpecularColor * s));
	}

	vec3 depth = ShadowCoord.xyz / ShadowCoord.w;
	float shadow = 1.0;

	if(ShadowCoord.x < 1.0 && ShadowCoord.x > 0 &&
		ShadowCoord.y < 1.0 && ShadowCoord.y > 0 &&
		ShadowCoord.z < 1.0 && ShadowCoord.z > 0) {
		
		vec2 moments = texture(shadowMap, depth.xy).xy;
	
		shadow = ChebychevInequality(moments, depth.z);
		shadow = VsmFixLightBleed(shadow, 0.1);
		shadow = clamp(shadow, 0.0,1.0);
	}

	color = clamp(tmpColor * shadow, 0.0, 1.0);

}